from persistent import Persistent

from zope.interface import implements
from zope.component import adapts

from zope.annotation.interfaces import IAnnotatable
from zope.annotation import factory

from collective.flowplayer.interfaces import IVideo, IAudio, IMediaInfo

class VideoInfo(Persistent):
    implements(IMediaInfo)
    adapts(IVideo)
    
    def __init__(self):
        self.height = None
        self.width = None
        self.audio_only = False

VideoInfoAdapter = factory(VideoInfo)

class AudioInfo(object):
    implements(IMediaInfo)
    adapts(IAudio)
    
    def __init__(self, context):
        self.audio_only = True
        self.width = None
        self.height = None


# This file is borrowed from the FlashVideo product, by Lukasz Lakomy


#########################################################
## Class that parses file header for FLV (Flash Videos)
## 
## Specification described here: http://osflash.org/flv
##
#########################################################

from struct import *

FLAGS     = {1:'video',
             4:'audio',
             5:'audio + video'}
TAG_TYPES = {'0x8':'audio',
             '0x9':'video',
             '0x12':'meta'}
AMF_TYPES = {'0x0':'number',
             '0x1':'boolean',
             '0x2':'string',
             '0x8':'mixed array',
             '0xb':'date',
             '0xc':'long string',
             }

class FLVHeaderError(Exception):
    pass

class FLVHeader:
    
    def __init__(self):
        self.height = None
        self.width = None
        
    def analyse(self, data):
        metatags = self.analyseContent(data)
        self.height = metatags.get("height",None)
        self.width = metatags.get("width",None)
        return metatags
    
    def bin2float(self, buffer):
        f = unpack(">d",buffer)
        return f[0]
    
    def getHeight(self):
        if self.height:
            return int(self.height)
        else:
            return None
    
    def getWidth(self):
        if self.width:
            return int(self.width)
        else:
            return None
    
    def getFlag(self, flag):
        if FLAGS.has_key(flag):
            return FLAGS[flag]
        else:
            return None
    
    def getTagType(self, tag):
        if TAG_TYPES.has_key(tag):
            return TAG_TYPES[tag]
        else:
            return None
        
    def getAmfType(self, amf_type):
        if AMF_TYPES.has_key(amf_type):
            return AMF_TYPES[amf_type]
        else:
            raise FLVHeaderError, "Unknown AMF Type %s"%amf_type
        
    def analyseContent(self,data):
        """
        """
        signature = data[:3] #always FLV
        if len(data)<=(8+17):
            raise FLVHeaderError,"Data size too small"    
        if not str(signature) == 'FLV':
            raise FLVHeaderError, "This does not appear to be an FLV file"
        metatags = {}
        version = ord(data[3]) #Currently 1 for known FLV files
        flag = ord(data[4]) # 5 - audio+video, 4 audio, 1 video 
        offset = ord(data[5])*(256**3) + ord(data[6])*(256)**2 + ord(data[7])*256 + ord(data[8]) #Total size of header (always 9 for known FLV files)
        header = {'signature':signature,
                  'version':version,
                  'flags':self.getFlag(flag),
                  'offset':offset}
        #print "header", header
        previousTag = ord(data[9])*(256**3) + ord(data[10])*(256)**2 + ord(data[11])*256 + ord(data[12])
        #print previousTag
        start = 13
        #FLV tag
        #print start
        tag_type = hex(ord(data[start]))
        tag_type = self.getTagType(tag_type)
        #print tag_type
        tag_size = ord(data[start+1])*(256)**2 + ord(data[start+2])*256 + ord(data[start+3]) #Size of Body (total tag size - 11)
        #print type(tag_size)
        timestamp = ord(data[start+4])*(256)**2 + ord(data[start+5])*256 + ord(data[start+6]) #Timestamp of tag (in milliseconds) 0
        timestamp2 = ord(data[start+7]) #Timestamp extension to form a uint32_be. This field has the upper 8 bits.
        stream_id = ord(data[start+8])*(256)**2 + ord(data[start+9])*256 + ord(data[start+10]) #Always 0
        s = start + 11
        #print s
        end = s + tag_size
        #print end
        body = data[s:end]
        meta_tag = {'type'         :tag_type,
                    'size'         :tag_size,
                    'timestamp'    :timestamp,
                    'ext_timestamp':timestamp2,
                    'stream_id'    :stream_id,
                    'body'         :body
                    }
        #print meta_tag
        amf1_start = 0
        #Each tag contains 2 AMF packets
        amf1_type = hex(ord(body[amf1_start]))
        amf1_type = self.getAmfType(amf1_type)
        if amf1_type != 'string':
            raise FLVHeaderError,"First AMF tag is not a string"
        amf1_len = ord(body[amf1_start+1])*256 + ord(body[amf1_start+2])
        #print amf1_len
        amf1_val = body[amf1_start+2:amf1_start+2+amf1_len+1]
        #print amf1_val
        amf2_start = amf1_start+2+amf1_len+1
        #print "amf2_start",amf2_start
        amf2_type = hex(ord(body[amf2_start]))
        amf2_type = self.getAmfType(amf2_type)
        #print "amf2_type",amf2_type
        if amf2_type != 'mixed array':
            raise FLVHeaderError,"Second AMF tag is not an array"
        max_index = ord(body[amf2_start+1])*(256**3) + ord(body[amf2_start+2])*(256**2) + ord(body[amf2_start+3])*(256) + ord(body[amf2_start+4])
        #print "max_index",max_index
        array_start = amf2_start+5
        for index in range(max_index):
            #print "\narray_start",array_start
            element_len = ord(body[array_start])*256 + ord(body[array_start+1])
            #print "element_len: ",element_len
            element_name = body[array_start+2:array_start+2+element_len]
            #print "element_name: ",element_name            
            value_type = hex(ord(body[array_start+element_len+2]))
            #print "value_type",value_type
            value_type = AMF_TYPES[value_type]
            #print "element_val_type",value_type
            if value_type == 'number':                
                element_val = body[array_start+element_len+2:array_start+element_len+2+9]
                value_value = element_val[1:] #8 bit double
                value_value = self.bin2float(value_value)
                #print "value_value",value_value
                array_start = array_start+element_len+2+9
            elif value_type == 'string':            
                value_len = ord(body[array_start+element_len+3])*256 + ord(body[array_start+element_len+4])
                #print "value_len: ",value_len
                value_value = str(body[array_start+element_len+5:array_start+element_len+5+value_len])
                #print "value_value",value_value
                array_start = array_start+element_len+5+value_len
            elif value_type == 'date':
                #I dont need that
                #Date is represented as 0x0B, then a double, then an int. 
                value_value = None
                array_start = array_start+3+element_len+8+2
            else:
                break
            metatags[element_name] = value_value
            
        if hex(ord(body[array_start])) == '0x0' and\
           hex(ord(body[array_start+1])) == '0x0' and\
           hex(ord(body[array_start+2])) == '0x9':
            #End of mixed array - OK
            pass
        else:
            pass
            #raise FLVHeaderError, "Wrong end of mixed array: %s%s%s"%(hex(ord(body[array_start])),
            #                                                          hex(ord(body[array_start+1])),
            #                                                          hex(ord(body[array_start+2])))
        return metatags


from zope.cachedescriptors import property
from zope.interface import alsoProvides, noLongerProvides
from zope.interface.interfaces import IInterface
from zope.component import getSiteManager

from collective.flowplayer.interfaces import IMediaInfo, IAudio, IVideo
from collective.flowplayer.flv import FLVHeader, FLVHeaderError

from Products.ATContentTypes import interface
from Products.Archetypes.interfaces import IObjectInitializedEvent

from StringIO import StringIO

VIDEO_EXTENSIONS = ['.f4b', '.f4p', '.f4v', '.flv', '.mp4', '.m4v', '.jpg', '.gif', '.png']
AUDIO_EXTENSIONS = ['.mp3']

def is_flowplayer_installed(object):
    sm = getSiteManager(context=object)
    return sm.queryUtility(IInterface, name=u'collective.flowplayer.interfaces.IFlowPlayerSite', default=False)

def remove_marker(object):
    changed = False
    if IAudio.providedBy(object):
        noLongerProvides(object, IAudio)
        changed = True
    if IVideo.providedBy(object):
        noLongerProvides(object, IVideo)
        changed = True
    if changed:
        object.reindexObject(idxs=['object_provides'])

class ChangeView(object):

    interface = None

    @property.Lazy
    def value(self):
        return self.content.getField('file').getRaw(self.content)

    def handleAudio(self):
        if not IAudio.providedBy(self.content):
            alsoProvides(self.content, IAudio)
            self.object.reindexObject(idxs=['object_provides'])

    def __init__(self, object, event):
        self.object = object
        # TODO: do we really need this different from object?
        self.content = content = event.object

        if not is_flowplayer_installed(content):
            return

        if not self.interface.providedBy(content):
            return

        if self.value is None:
            remove_marker(content)
            return

        ext = self.check_extension()
        if ext is None:
            remove_marker(content)
            return

        if IObjectInitializedEvent.providedBy(event):
            content.setLayout('flowplayer')

        if ext in AUDIO_EXTENSIONS:
            self.handleAudio()
        elif ext in VIDEO_EXTENSIONS:
            self.handleVideo()

    def handleVideo(self):
        if not IVideo.providedBy(self.content):
            alsoProvides(self.content, IVideo)
            self.object.reindexObject(idxs=['object_provides'])

class ChangeFileView(ChangeView):

    interface = interface.IFileContent

    def check_extension(self):
        filename = self.value.filename.lower()
        for ext in AUDIO_EXTENSIONS + VIDEO_EXTENSIONS:
            if filename.endswith(ext):
                return ext
        return None

    def handleVideo(self):
        file_object = self.value
        try:
            # For blobs
            file_handle = file_object.getIterator()
        except AttributeError:
            file_handle = StringIO(str(file_object.data))

        file_handle.seek(0)
        height = width = None
        flvparser = FLVHeader()
        try:
            flvparser.analyse(file_handle.read(1024))
            width = flvparser.getWidth()
            height = flvparser.getHeight()
        except FLVHeaderError:
            # Do not remove marker interface. MP4 files can't be parsed 
            # but works fine. Any file which extension is file can be 
            # played (hopefully)
            # remove_marker(self.content)
            # return
            pass

        super(ChangeFileView, self).handleVideo()

        if height and width:
            info = IMediaInfo(self.content)
            info.height = height
            info.width = width

class ChangeLinkView(ChangeView):

    interface = interface.IATLink

    @property.Lazy
    def value(self):
        return self.content.getField('remoteUrl').getRaw(self.content)

    def check_extension(self):
        filename = self.value.lower()
        for ext in AUDIO_EXTENSIONS + VIDEO_EXTENSIONS:
            if filename.endswith(ext):
                return ext
        return None



import unittest
from Testing import ZopeTestCase

from OFS.PropertyManager import PropertyManager
from collective.flowplayer.utils import properties_to_dict, flash_properties_to_dict

class TestUtils(ZopeTestCase.ZopeTestCase):
    
    def afterSetUp(self):
        self.props = PropertyManager()
        # PropertyManager has title property already
        self.props.manage_changeProperties(title = 'Player properties')
        self.props.manage_addProperty('plugins/controls/url', '${portal_path}flowplayer.controls.swf', 'string')
        self.props.manage_addProperty('plugins/controls/all', False, 'boolean')
        self.props.manage_addProperty('plugins/controls/play', True, 'boolean')
        self.props.manage_addProperty('plugins/controls/scrubber', True, 'boolean')
        self.props.manage_addProperty('plugins/controls/tooltips/fullscreen', 'Enter fullscreen mode', 'string')
        self.props.manage_addProperty('plugins/controls/tooltips/buttons', True, 'boolean')
        self.props.manage_addProperty('plugins/audio/url', '${portal_url}++resource++collective.flowplayer/flowplayer.audio.swf', 'string')
        self.props.manage_addProperty('clip/autoPlay',False, 'boolean')
        self.props.manage_addProperty('clip/autoBuffering', True, 'boolean')
        self.props.manage_addProperty('param/src', 'flowplayer.swf', 'string')
        self.props.manage_addProperty('param/wmode', 'opaque', 'string')
    
    def test_parsing(self):
        parsed = properties_to_dict(self.props, 'http://localhost', ignore=['title'])
        self.assertEqual(len(parsed.keys()), 2) # plugins, clip
        self.assertEqual(len(parsed['plugins'].keys()), 2) # controls, audio
        self.assertEqual(parsed['plugins']['controls']['all'], False) 
        self.assertEqual(parsed['plugins']['controls']['scrubber'], True) 
        self.assertEqual(parsed['plugins']['controls']['url'], r'http%3A//localhost/flowplayer.controls.swf') 
        self.assertEqual(parsed['plugins']['audio']['url'], r'http%3A//localhost/%2B%2Bresource%2B%2Bcollective.flowplayer/flowplayer.audio.swf') 
        self.assertEqual(parsed['clip']['autoBuffering'], True) 
        self.failIf(parsed.has_key('param'))
        
    def test_parsing_flash_props(self):
        parsed = flash_properties_to_dict(self.props, 'http://localhost')
        self.assertEqual(len(parsed.keys()), 2) # src, wmode
        self.assertEqual(parsed['src'], 'flowplayer.swf')
        self.assertEqual(parsed['wmode'], 'opaque')

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestUtils))
    return suite


from zope import interface
from zope import component
from Acquisition import aq_inner
import simplejson
import urllib
import os 

from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName
from collective.flowplayer.utils import properties_to_dict, \
                                        flash_properties_to_dict

from collective.flowplayer.interfaces import IFlowPlayable
from collective.flowplayer.interfaces import IMediaInfo, IFlowPlayerView

from plone.memoize.instance import memoize
from plone.memoize import view

class JavaScript(BrowserView):
    
    @view.memoize_contextless
    def portal_state(self):
        """ returns 
            http://dev.plone.org/plone/browser/plone.app.layout/trunk/plone/app/layout/globals/portal.py
        """
        return component.getMultiAdapter((self.context, self.request), name=u"plone_portal_state")

    @property
    def flowplayer_properties(self):
        properties_tool = getToolByName(self.context, 'portal_properties')
        return getattr(properties_tool, 'flowplayer_properties', None)

    @property
    def flowplayer_properties_as_dict(self):
        portal_url = self.portal_state().portal_url()
        return properties_to_dict(self.flowplayer_properties, 
                                  portal_url, 
                                  ignore=['title', 
                                          'loop',
                                          'initialVolumePercentage',
                                          'showPlaylist',])

    @property
    def flash_properties_as_dict(self):
        portal_url = self.portal_state().portal_url()
        return flash_properties_to_dict(self.flowplayer_properties, 
                                        portal_url)

    def update(self):
        portal_url = self.portal_state().portal_url()
        if not portal_url.endswith('/'):
            portal_url += '/'

        # if showPlaylist is True, do not show playlist buttons on controlbar
        self.show_cb_playlist_buttons = not self.flowplayer_properties.getProperty('showPlaylist')
        # build string in Javascript format which is appended to the player
        # It contains javascript events which can't be configured in the 
        # self.properties, because simplejson can't handle them
        self.events = ''
        volume = self.flowplayer_properties.getProperty('initialVolumePercentage')
        if volume:
            self.events += '.onLoad( function() { this.setVolume(%d); })' % volume
        if self.flowplayer_properties.getProperty('loop'):
            self.events += '.onBeforeFinish( function() { return false; })'

        
    def __call__(self, request=None, response=None):
        """ Returns global configuration of the Flowplayer taken from portal_properties """
        self.update()
        self.request.response.setHeader("Content-type", "text/javascript")
            
        return """(function($) {
        $(function() { 
            $('.autoFlowPlayer').each(function() {
                var config = %(config)s;
                if ($(this).is('.minimal')) { config.plugins.controls = null; }
                var audio = $(this).is('.audio');
                if (audio) {
                    $(this).width(500);
                    config.plugins.controls.all = false;
                    config.plugins.controls.play = true;
                    config.plugins.controls.scrubber = true;
                    config.plugins.controls.mute = true;
                    config.plugins.controls.volume = true;
                }
                if ($(this).is('div')) {
                    // comming from Kupu, there are relative urls
                    config.clip.baseUrl = $('base').attr('href');
                    config.clip.url = $(this).find('a').attr('href');
                    // Ignore global autoplay settings
                    if ($(this).find('img').length == 0) {
                        // no image. Don't autoplay, remove all elements inside the div to show player directly.
                        config.clip.autoPlay = false;
                        $(this).empty();
                    } else {
                        // Clip is probably linked as image, so autoplay the clip after image is clicked
                        config.clip.autoPlay = true;
                    }
                }
                flowplayer(this, %(params)s, config)%(events)s;
                $('.flowPlayerMessage').remove();
            });
            $('.playListFlowPlayer').each(function() {
                var config = %(config)s;
                var audio = $(this).is('.audio');
                if (audio) { config.plugins.controls.fullscreen = false; }
                if ($(this).is('.minimal')) { config.plugins.controls = null; }
                if ($(this).find('img').length > 0) { 
                    // has splash
                    config.clip.autoPlay = true;
                }
                portlet_parents = $(this).parents('.portlet');
                var playlist_selector = 'div#flowPlaylist';
                if (portlet_parents.length > 0) {
                    var portlet = true;
                    // playlist has to be bound to unique item
                    playlist_selector_id = portlet_parents.parent().attr('id')+'-playlist';
                    $(this).parent().find('.flowPlaylist-portlet-marker').attr('id', playlist_selector_id);
                    playlist_selector = '#'+playlist_selector_id;
                    if (audio) {
                        config.plugins.controls.all = false;
                        config.plugins.controls.play = true;
                        config.plugins.controls.scrubber = true;
                        config.plugins.controls.mute = true;
                        config.plugins.controls.volume = false;
                    }
                } else {
                    var portlet = false;
                }
                if (!portlet) {
                    $("#pl").scrollable({items:playlist_selector, size:4, clickable:false});
                }
                // manual = playlist is setup using HTML tags, not using playlist array in config
                flowplayer(this, %(params)s, config).playlist(playlist_selector, {loop: true, manual: true})%(events)s;
                $(this).show();
                $('.flowPlayerMessage').remove();

            });
        });
})(jQuery);
""" % dict(params = simplejson.dumps(self.flash_properties_as_dict),
           config = simplejson.dumps(self.flowplayer_properties_as_dict, indent=4),
           events = self.events,
          )

class File(BrowserView):
    interface.implements(IFlowPlayerView)
    
    def __init__(self, context, request):
        super(File, self).__init__(context, request)
        
        self.info = IMediaInfo(self.context, None)
        
        self.height = self.info is not None and self.info.height or None
        self.width = self.info is not None and self.info.width or None
        self._audio_only = self.info is not None and self.info.audio_only or None
        
        if self.height and self.width:
            self._scale = "height: %dpx; width: %dpx;" % (self.height, self.width)
        else:
            self._scale = ""
    
    def audio_only(self):
        return self._audio_only
    
    def scale(self):
        return self._scale
    
    def videos(self):
        return[dict(url=self.href(),
                    title=self.context.Title(),
                    description=self.context.Description(),
                    height=self.height,
                    width=self.width,
                    audio_only=self._audio_only)]

    def href(self):
        context = aq_inner(self.context)
        ext = ''
        url = self.context.absolute_url()
        filename = context.getFilename()
        if filename:
            extension = os.path.splitext(filename)[1]
            if not url.endswith(extension):
                ext = "?e=%s" % extension
        return self.context.absolute_url()+ext

class Link(File):

    def href(self):
        return self.context.getRemoteUrl()

class Folder(BrowserView):
    interface.implements(IFlowPlayerView)
    
    @memoize
    def playlist_class(self):
        properties_tool = getToolByName(self.context, 'portal_properties')
        props = getattr(properties_tool, 'flowplayer_properties', None)
        return props.getProperty('showPlaylist') and 'flowPlaylistVisible' or 'flowPlaylistHidden'
        
    @memoize
    def audio_only(self):
        return len([v for v in self.videos() if not v['audio_only']]) == 0
    
    @memoize
    def scale(self):
        height = 0
        width = 0
        if self.audio_only():
            height = 27
            width = 400
        
        for video in self.videos():
            if video['height'] > height or video['width'] > width:
                height = video['height']
                width = video['width']
                
        if height and width:
            return "height: %dpx; width: %dpx;" % (height, width)
    
    @memoize
    def videos(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        results = []
        for brain in self._query():
            video = brain.getObject()
            if not IFlowPlayable.providedBy(video):
                continue
            view = component.getMultiAdapter(
                (video, self.request), interface.Interface, 'flowplayer')
            results.append(dict(url=view.href(),
                                title=brain.Title,
                                description=brain.Description,
                                height=view.height,
                                width=view.width,
                                audio_only=view.audio_only()))
        return results

    def first_clip_url(self):
        """ Clip must be quoted to playlist is able to find it in the flowplayer-playlist onBegin/getEl method call """
        videos = self.videos()
        if videos:
            return urllib.quote(videos[0].get('url'))
        else:
            return None
    
    def _query(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        return catalog(object_provides=IFlowPlayable.__identifier__,
                       path = {'depth': 1, 'query': '/'.join(self.context.getPhysicalPath())},
                       sort_on='getObjPositionInParent')

class Topic(Folder):
    interface.implements(IFlowPlayerView)
    
    def _query(self):
        return self.context.queryCatalog()



import unittest
from Products.CMFCore.utils import getToolByName
from Products.PloneTestCase.PloneTestCase import FunctionalTestCase
from collective.flowplayer.utils import properties_to_dict
from collective.flowplayer.migration import migrateTo30
from collective.flowplayer.testing import layer
        
class TestMigration30(FunctionalTestCase):
    layer = layer
    
    def afterSetUp(self):
        self.props = getToolByName(self.portal, 'portal_properties').flowplayer_properties
        # add old properties, defined in previous versions of collective.flowplayer
        self.props.manage_addProperty('autoPlay', False, 'boolean')
        self.props.manage_addProperty('autoBuffering', True, 'boolean')
        self.props.manage_addProperty('initialScale', 'fit', 'string')
        self.props.manage_addProperty('showVolumeSlider', False, 'boolean')
        self.props.manage_addProperty('controlBarGloss', False, 'boolean')
        self.props.manage_addProperty('useNativeFullScreen', True, 'boolean')

    def test_migration(self):
        # assert values defined in propertiestool.xml
        self.assertEqual(self.props.getProperty('clip/autoPlay'), False)
        self.assertEqual(self.props.getProperty('clip/autoBuffering'), False)
        self.assertEqual(self.props.getProperty('plugins/controls/volume'), True)
        self.assertEqual(self.props.getProperty('clip/scaling'), 'fit')
        migrateTo30(self.portal)
        # check values are migrated
        self.assertEqual(self.props.getProperty('clip/autoPlay'), False)
        self.assertEqual(self.props.getProperty('clip/autoBuffering'), True)
        self.assertEqual(self.props.getProperty('clip/scaling'), 'fit')
        self.assertEqual(self.props.getProperty('plugins/controls/volume'), False)
        # old properties has to be removed now
        self.failIf(self.props.hasProperty('autoPlay'))
        self.failIf(self.props.hasProperty('autoBuffering'))
        self.failIf(self.props.hasProperty('showVolumeSlider'))
        self.failIf(self.props.hasProperty('initalScale'))
        # Those are deleted because not migrated at all
        self.failIf(self.props.hasProperty('controlBarGloss'))
        self.failIf(self.props.hasProperty('useNativeFullScreen'))

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestMigration30))
    return suite


from zope.interface import implements
from zope.component import getMultiAdapter, queryMultiAdapter

from plone.portlets.interfaces import IPortletDataProvider
from plone.app.portlets.portlets import base
import urllib
import random
from zope import schema
from zope.formlib import form

from plone.memoize.instance import memoize

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.app.vocabularies.catalog import SearchableTextSourceBinder
from plone.app.form.widgets.uberselectionwidget import UberSelectionWidget

from Products.ATContentTypes.interface import IATTopic, IATFolder, IATImage

from collective.flowplayer.interfaces import IFlowPlayable
from collective.flowplayer.interfaces import IFlowPlayerView
from collective.flowplayer import MessageFactory as _

from Products.CMFCore.utils import getToolByName

class IVideoPortlet(IPortletDataProvider):
    """A portlet which can display videos
    """

    header = schema.TextLine(title=_(u"Portlet header"),
                             description=_(u"Title of the rendered portlet"),
                             required=True)

    target = schema.Choice(title=_(u"Target object"),
                           description=_(u"This can be a file containing an FLV file, or a folder or collection containing FLV files"),
                           required=True,
                           source=SearchableTextSourceBinder({'object_provides' : [IATTopic.__identifier__,
                                                                                   IATFolder.__identifier__,
                                                                                   IFlowPlayable.__identifier__]},
                                                               default_query='path:'))
                                                               
    splash = schema.Choice(title=_(u"Splash image"),
                           description=_(u"An image file to use as a splash image"),
                           required=False,
                           source=SearchableTextSourceBinder({'object_provides' : [IATImage.__identifier__, IATFolder.__identifier__,]},
                                                               default_query='path:'))

    limit = schema.Int(title=_(u"Number of videos to show"),
                       description=_(u"Enter a number greater than 0 to limit the number of items displayed"),
                       required=False)
    
    random = schema.Bool(title=_(u"Randomise the playlist"),
                         description=_(u"If enabled, a random video from the selection will be played."),
                         required=True,
                         default=False)
                       
    show_more = schema.Bool(title=_(u"Show more... link"),
                       description=_(u"If enabled, a more... link will appear in the footer of the portlet, "
                                      "linking to the underlying data."),
                       required=True,
                       default=True)

class Assignment(base.Assignment):
    implements(IVideoPortlet)

    header = u""
    target = None
    splash = None
    limit = None
    random = False
    show_more = True

    def __init__(self, header=u"", target=None, splash=None, limit=None, random=False, show_more=True):
        self.header = header
        self.target = target
        self.splash = splash
        self.limit = limit
        self.random = random
        self.show_more = show_more

    @property
    def title(self):
        return self.header

class Renderer(base.Renderer):
    render = ViewPageTemplateFile('portlet.pt')

    @property
    def available(self):
        return len(self.videos()) > 0

    def target_url(self):
        target = self.target()
        if target is None:
            return None
        else:
            return target.absolute_url()
    
    @memoize
    def splash(self):
        splash_path = self.data.splash
        if not splash_path:
            return None

        if splash_path.startswith('/'):
            splash_path = splash_path[1:]
        
        if not splash_path:
            return None

        portal_state = getMultiAdapter((self.context, self.request), name=u'plone_portal_state')
        portal = portal_state.portal()
        splash = portal.restrictedTraverse(splash_path, default=None)
        
        if splash is not None and not IATImage.providedBy(splash):
            return None
        
        return splash

    @memoize
    def videos(self):
        
        target = self.target()
        catalog = getToolByName(self.context, 'portal_catalog')
        
        if target is None:
            return []
        
        view = queryMultiAdapter((target, self.request), name=u"flowplayer")
        if view is None or not IFlowPlayerView.providedBy(view):
            return []
            
        videos = view.videos()
        
        limit = self.data.limit
        if limit:
            result = videos[:limit]
        else:
            result = videos

        if self.data.random:
            random.shuffle(result)
        return result


    def first_clip_url(self):
        """ Clip must be quoted to playlist is able to find it in the flowplayer-playlist onBegin/getEl method call """
        videos = self.videos()
        if videos:
            return urllib.quote(videos[0].get('url'))
        else:
            return None
            
    @memoize
    def audio_only(self):
        target = self.target()
        view = queryMultiAdapter((target, self.request), name=u"flowplayer")
        if view is None or not IFlowPlayerView.providedBy(view):
            return False
        return view.audio_only()
        
    @memoize
    def scale(self):
        target = self.target()
        view = queryMultiAdapter((target, self.request), name=u"flowplayer")
        if view is None or not IFlowPlayerView.providedBy(view):
            return False
        return view.scale()

    @memoize
    def target(self):
        target_path = self.data.target
        if not target_path:
            return None

        if target_path.startswith('/'):
            target_path = target_path[1:]
        
        if not target_path:
            return None

        portal_state = getMultiAdapter((self.context, self.request), name=u'plone_portal_state')
        portal = portal_state.portal()
        return portal.restrictedTraverse(target_path, default=None)
        
class AddForm(base.AddForm):
    form_fields = form.Fields(IVideoPortlet)
    form_fields['target'].custom_widget = UberSelectionWidget
    form_fields['splash'].custom_widget = UberSelectionWidget
    
    label = _(u"Add Video Portlet")
    description = _(u"This portlet display a Flash Video")

    def create(self, data):
        return Assignment(**data)

class EditForm(base.EditForm):
    form_fields = form.Fields(IVideoPortlet)
    form_fields['target'].custom_widget = UberSelectionWidget
    form_fields['splash'].custom_widget = UberSelectionWidget

    label = _(u"Edit Video Portlet")
    description = _(u"This portlet display a Flash video.")



from Products.PloneTestCase import ptc

from collective.testcaselayer import ptc as tcl_ptc

ptc.setupPloneSite()

class Layer(tcl_ptc.BasePTCLayer):
    """Install collective.flowplayer"""

    def afterSetUp(self):
        self.addProfile('collective.flowplayer:default')
        # put resource registry to debug mode to avoid cachekyes in tests
        self.portal.portal_css.setDebugMode(True)
        self.portal.portal_javascripts.setDebugMode(True)

layer = Layer([tcl_ptc.ptc_layer])



from setuptools import setup, find_packages
import os

version = '3.0b4'

tests_require = ['collective.testcaselayer']

setup(name='collective.flowplayer',
      version=version,
      description="A simple package using Flowplayer for video content",
      long_description=open("README.txt").read() + "\n\n" +
                       open(os.path.join("docs", "UPGRADE.txt")).read() + "\n\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='flv flash video plone',
      author='Martin Aspeli',
      author_email='optilude@gmail.com',
      url='http://plone.org',
      license='GPL',
      packages=find_packages('src', exclude=['ez_setup']),
      package_dir = {'':'src'},
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'simplejson',
          'plone.app.jquerytools',
          # 'Plone', Too confusing for those using releases < 3.2
      ],
      tests_require=tests_require,
      extras_require={'tests': tests_require},
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



# Migration utilities and migration steps
from Products.CMFCore.utils import getToolByName
import logging
logger  = logging.getLogger('flowplayer-migration')

# oldProperty: newProperty. do not properties which name is not changed (e.g. player)
V30_PROPERTIES_MAPPING = {
    'autoPlay'          : 'clip/autoPlay',
    'autoBuffering'     : 'clip/autoBuffering',
    'initialScale'      : 'clip/scaling',
    'showVolumeSlider'  : 'plugins/controls/volume',
    'controlBarBackgroundColor' : 'plugins/controls/backgroundColor',
}

V30_TO_DELETE = [
    'usePlayOverlay', 
    'useNativeFullScreen', 
    'controlsOverVideo',
    'controlBarGloss',
]
def emptyMigrate(self):
    pass

def migrateTo30(context):
    properties_tool = getToolByName(context, 'portal_properties', None)
    if properties_tool is not None:
        flowplayer_properties = getattr(properties_tool, 'flowplayer_properties', None)
        if flowplayer_properties:
            for k, v in V30_PROPERTIES_MAPPING.items():
                if flowplayer_properties.hasProperty(k) and flowplayer_properties.hasProperty(v):
                        # has both old and new property. This is after installation, without
                        # migration run. Migrate!
                        flowplayer_properties._updateProperty(v, flowplayer_properties.getProperty(k))
                        flowplayer_properties.manage_delProperties([k])
                        logger.info('Property "%s" has been migrated to "%s"', k, v)
                # ignore any other combinations, because these are not expected
            for k in V30_TO_DELETE:
                if flowplayer_properties.hasProperty(k):
                    flowplayer_properties.manage_delProperties([k])
                    logger.info('Property "%s" has been removed', k)

def migrateTo30b2(context):
    properties_tool = getToolByName(context, 'portal_properties', None)
    if properties_tool is not None:
        flowplayer_properties = getattr(properties_tool, 'flowplayer_properties', None)
        if flowplayer_properties:
            if flowplayer_properties.hasProperty('player'):
                flowplayer_properties._updateProperty('param/src', flowplayer_properties.getProperty('player'))
                flowplayer_properties.manage_delProperties(['player'])
                logger.info('Property "player" has been migrated to "param/src"')



from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IPropertiesTool

# Properties are defined here, because if they are defined in propertiestool.xml,
# all properties are re-set the their initial state if you reinstall product
# in the quickinstaller.

_PROPERTIES = [
    dict(name='param/src', type_='string', value='${portal_url}/++resource++collective.flowplayer/flowplayer.swf'),
    dict(name='loop', type_='boolean', value=False),
    dict(name='showPlaylist', type_='boolean', value=True),
    dict(name='initialVolumePercentage', type_='int', value=50),
    dict(name='plugins/controls/url', type_='string', value='${portal_url}/++resource++collective.flowplayer/flowplayer.controls.swf'),
    dict(name='plugins/audio/url', type_='string', value='${portal_url}/++resource++collective.flowplayer/flowplayer.audio.swf'),
    dict(name='clip/autoPlay', type_='boolean', value=False),
    dict(name='clip/autoBuffering', type_='boolean', value=False),
    dict(name='clip/scaling', type_='string', value='fit'),
    dict(name='plugins/controls/volume', type_='boolean', value=True),
]

def import_various(context):
    
    if not context.readDataFile('collective.flowplayer.txt'):
        return
        
    site = context.getSite()
    kupu = getToolByName(site, 'kupu_library_tool')
    
    paragraph_styles = list(kupu.getParagraphStyles())
    
    new_styles = [('autoFlowPlayer video', 'Video|div'),
                  ('autoFlowPlayer video image-left', 'Video (left)|div'),
                  ('autoFlowPlayer video image-right', 'Video (right)|div'),
                  ('autoFlowPlayer audio', 'Audio|div'),
                  ('autoFlowPlayer audio image-left', 'Audio (left)|div'),
                  ('autoFlowPlayer audio image-right', 'Audio (right)|div')]
    to_add = dict(new_styles)
    
    for style in paragraph_styles:
        css_class = style.split('|')[-1]
        if css_class in to_add:
            del to_add[css_class]

    if to_add:
        paragraph_styles += ['%s|%s' % (v, k) for k,v in new_styles if k in to_add]
        kupu.configure_kupu(parastyles=paragraph_styles)
        
    # Define portal properties
    ptool = getToolByName(site, 'portal_properties')
    props = ptool.flowplayer_properties

    # We don't want to migrate contents of 'player' property. Check if there is 
    # 'plugins/controls/url' property (which indicates we migrated to new version
    # already) and if there is not one, remove 'player' property to force new
    # version of this property
    if not props.hasProperty('plugins/controls/url'):
        if props.hasProperty('player'):
            props.manage_delProperties(['player'])
    
    for prop in _PROPERTIES:
        if not props.hasProperty(prop['name']):
            props.manage_addProperty(prop['name'], prop['value'], prop['type_'])
    



import unittest
from zope.testing import doctest

from Testing import ZopeTestCase
from Products.PloneTestCase import ptc

from collective.flowplayer import testing

optionflags = (doctest.NORMALIZE_WHITESPACE|
               doctest.ELLIPSIS|
               doctest.REPORT_NDIFF)

def test_suite():
    suite = ZopeTestCase.FunctionalDocFileSuite(
        'README.txt',
        package='collective.flowplayer',
        optionflags=optionflags,
        test_class=ptc.FunctionalTestCase)
    suite.layer = testing.layer
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



from zope.interface import Interface
from zope import schema

from collective.flowplayer import MessageFactory as _

class IFlowPlayerSite(Interface):
    """Marker interface for sites with this product installed"""

class IFlowPlayerView(Interface):
    """View for the flow player
    """
    
    def audio_only():
        """Return True if we are only showing audio files.
        """
    
    def scale():
        """Return a CSS/style snippet to encoding the height and width of
        the player.
        """
    
    def videos():
        """Return a list of dicts for videos to play, with keys url, title,
        description and scale.
        """

class IFlowPlayable(Interface):
    """A file playable in the flowplayer
    """

class IVideo(IFlowPlayable):
    """Marker interface for files that contain FLV content
    """
    
class IAudio(IFlowPlayable):
    """Marker interface for files that contain audio content
    """
    
class IMediaInfo(IVideo):
    """Information about a video object
    """
    
    audio_only = schema.Bool(title=u"Audio only?", required=True)
    
    width = schema.Int(title=_(u"Width"), required=False)
    height = schema.Int(title=_(u"Height"), required=False)


import urllib

def flash_properties_to_dict(propertysheet, portal_url):
    items = dict()
    
    if not portal_url.endswith('/'):
        portal_url += '/'
    # build python representation of properties first
    for key, value in propertysheet.propertyItems():
        if key.startswith('param/'):
            # process param/ properties only
            if isinstance(value, str):
                new_value = value.replace('${portal_path}', portal_url)
                new_value = new_value.replace('${portal_url}', portal_url)
            else:
                new_value = value
            # key name is param/src - there must be 'src' only in result dict
            items[key[6:]] = new_value
    return items
            

def properties_to_dict(propertysheet, portal_url, ignore=['title']):
    """
    Analyses portal properties and creates python dictionary from keys-values. 
    Key in the form 'k1/k2/k3' having a value 'value' is transformed to dictionary:
    items = { k1 : { k2 : { k3 : value } } }
    """
    items = dict()
    
    if not portal_url.endswith('/'):
        portal_url += '/'
    
    # build python representation of properties first
    for key, value in propertysheet.propertyItems():
        if key in ignore:
            continue
            
        # automatically ignore all properties starting with 'param/' - these are flash properties
        if key.startswith('param/'):
            continue
            
        if isinstance(value, str):
            new_value = value.replace('${portal_path}', portal_url)
            new_value = new_value.replace('${portal_url}', portal_url)
        else:
            new_value = value
            
        # quote any key with /url in it, because we can't pass ++resource++ 
        # to the flash as argument - it will replace + with space and file 
        # is not found.
        if key.endswith('/url'):
            new_value = urllib.quote(new_value)

        keys = key.split('/')
        to_fill = items
        for idx, k in enumerate(keys):
            if not k:
                continue
            if not to_fill.has_key(k):
                if idx < len(keys)-1:
                    to_fill[k] = dict()
                    to_fill = to_fill[k]
                else:
                    to_fill[k] = new_value
            else:
                to_fill = to_fill[k]

    return items
